@@ -36,7 +36,6 @@ from typing import Any, Dict, List, Optional, Sequence, Tuple, Union, ValuesView
 from .charsets import MYSQL_CHARACTER_SETS, MYSQL_CHARACTER_SETS_57
 from .errors import ProgrammingError
 
-MAX_PACKET_LENGTH: int = 16777215
 NET_BUFFER_LENGTH: int = 8192
 MAX_MYSQL_TABLE_COLUMNS: int = 4096
 # Flag used to send the Query Attributes with 0 (or more) parameters.
@@ -733,7 +732,7 @@ class CharacterSet(_Constants):
             version (tuple): MySQL version tuple.
         """
         cls.mysql_version = version[:2]
-        if cls.mysql_version == (8, 0):
+        if cls.mysql_version >= (8, 0):
             cls.desc = MYSQL_CHARACTER_SETS
 
     @classmethod
@@ -816,7 +815,7 @@ class CharacterSet(_Constants):
             except IndexError as err:
                 raise ProgrammingError(f"Character set ID {charset} unknown") from err
 
-        if charset in ("utf8", "utf-8") and cls.mysql_version == (8, 0):
+        if charset in ("utf8", "utf-8") and cls.mysql_version >= (8, 0):
             charset = "utf8mb4"
         if charset is not None and collation is None:
             info = cls.get_default_collation(charset)
