@@ -1,4 +1,4 @@
-# Copyright (c) 2014, 2022, Oracle and/or its affiliates.
+# Copyright (c) 2014, 2023, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -320,7 +320,7 @@ class CMySQLCursor(MySQLCursorAbstract):
                     stmt = stmt.replace(f"%({key})s".encode(), value)
             elif isinstance(prepared, (list, tuple)):
                 psub = _ParamSubstitutor(prepared)
-                stmt = RE_PY_PARAM.sub(psub, stmt)
+                stmt = RE_PY_PARAM.sub(psub, stmt)  # type: ignore[call-overload]
                 if psub.remaining != 0:
                     raise ProgrammingError(
                         "Not all parameters were used in the SQL statement"
@@ -388,7 +388,7 @@ class CMySQLCursor(MySQLCursorAbstract):
                         tmp = tmp.replace(f"%({key})s".encode(), value)
                 elif isinstance(prepared, (list, tuple)):
                     psub = _ParamSubstitutor(prepared)
-                    tmp = RE_PY_PARAM.sub(psub, tmp)
+                    tmp = RE_PY_PARAM.sub(psub, tmp)  # type: ignore[call-overload]
                     if psub.remaining != 0:
                         raise ProgrammingError(
                             "Not all parameters were used in the SQL statement"
@@ -806,9 +806,7 @@ class CMySQLCursorBuffered(CMySQLCursor):
             row = self._rows[self._next_row]
         except IndexError:
             return None
-        else:
-            self._next_row += 1
-
+        self._next_row += 1
         return row
 
     def fetchall(self) -> List[RowType]:
@@ -976,7 +974,7 @@ class CMySQLCursorNamedTuple(CMySQLCursor):
         res = super().fetchmany(size=size)
         if not res:
             return []
-        return [self.named_tuple(*res[0])]
+        return [self.named_tuple(*row) for row in res]
 
     def fetchall(self) -> List[RowType]:
         """Return all rows of a query result set.
@@ -1278,3 +1276,13 @@ class CMySQLCursorPreparedDict(CMySQLCursorDict, CMySQLCursorPrepared):  # type:
     3. CMySQLCursorPrepared (right parent class)
     4. CMySQLCursor (base class)
     """
+
+
+class CMySQLCursorPreparedNamedTuple(CMySQLCursorNamedTuple, CMySQLCursorPrepared):
+    """This class is a blend of features from CMySQLCursorNamedTuple and CMySQLCursorPrepared"""
+
+
+class CMySQLCursorPreparedRaw(CMySQLCursorPrepared):
+    """This class is a blend of features from CMySQLCursorRaw and CMySQLCursorPrepared"""
+
+    _raw: bool = True
